package resources

import (
	"errors"
	"fmt"
	"time"

	"cloud.o-forge.io/core/oc-lib/models/common/pricing"
	"cloud.o-forge.io/core/oc-lib/tools"
)

type PricedResource struct {
	Name                     string                      `json:"name,omitempty" bson:"name,omitempty"`
	Logo                     string                      `json:"logo,omitempty" bson:"logo,omitempty"`
	InstancesRefs            map[string]string           `json:"instances_refs,omitempty" bson:"instances_refs,omitempty"`
	PricingProfiles          []pricing.PricingProfileITF `json:"pricing_profiles,omitempty" bson:"pricing_profiles,omitempty"`
	SelectedPricing          pricing.PricingProfileITF   `json:"selected_pricing,omitempty" bson:"selected_pricing,omitempty"`
	ExplicitBookingDurationS float64                     `json:"explicit_location_duration_s,omitempty" bson:"explicit_location_duration_s,omitempty"`
	UsageStart               *time.Time                  `json:"start,omitempty" bson:"start,omitempty"`
	UsageEnd                 *time.Time                  `json:"end,omitempty" bson:"end,omitempty"`
	CreatorID                string                      `json:"peer_id,omitempty" bson:"peer_id,omitempty"`
	ResourceID               string                      `json:"resource_id,omitempty" bson:"resource_id,omitempty"`
	ResourceType             tools.DataType              `json:"resource_type,omitempty" bson:"resource_type,omitempty"`
}

func (abs *PricedResource) GetID() string {
	return abs.ResourceID
}

func (abs *PricedResource) GetType() tools.DataType {
	return abs.ResourceType
}

func (abs *PricedResource) GetCreatorID() string {
	return abs.CreatorID
}

func (abs *PricedResource) IsPurchased() bool {
	if abs.SelectedPricing == nil {
		return false
	}
	return (abs.SelectedPricing).IsPurchased()
}

func (abs *PricedResource) GetLocationEnd() *time.Time {
	return abs.UsageEnd
}

func (abs *PricedResource) GetLocationStart() *time.Time {
	return abs.UsageStart
}

func (abs *PricedResource) SetLocationStart(start time.Time) {
	abs.UsageStart = &start
}

func (abs *PricedResource) SetLocationEnd(end time.Time) {
	abs.UsageEnd = &end
}

func (abs *PricedResource) GetExplicitDurationInS() float64 {
	if abs.ExplicitBookingDurationS == 0 {
		if abs.UsageEnd == nil && abs.UsageStart == nil {
			return time.Duration(1 * time.Hour).Seconds()
		}
		if abs.UsageEnd == nil {
			add := abs.UsageStart.Add(time.Duration(1 * time.Hour))
			abs.UsageEnd = &add
		}
		return abs.UsageEnd.Sub(*abs.UsageStart).Seconds()
	}
	return abs.ExplicitBookingDurationS
}

func (r *PricedResource) GetPrice() (float64, error) {
	fmt.Println("GetPrice", r.UsageStart, r.UsageEnd)
	now := time.Now()
	if r.UsageStart == nil {
		r.UsageStart = &now
	}
	if r.UsageEnd == nil {
		add := r.UsageStart.Add(time.Duration(1 * time.Hour))
		r.UsageEnd = &add
	}
	if r.SelectedPricing == nil {
		if len(r.PricingProfiles) == 0 {
			return 0, errors.New("pricing profile must be set on Priced Resource " + r.ResourceID)
		}
		r.SelectedPricing = r.PricingProfiles[0]
	}
	pricing := r.SelectedPricing
	return pricing.GetPrice(1, 0, *r.UsageStart, *r.UsageEnd)
}
